;;; emacsia-block.el --- Block-system for Emacsia

;; Copyright (C) 2004  Free Software Foundation, Inc.

;; Author: Ulrik Jensen <terryp@daimi.au.dk>
;; Keywords: 

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(defvar emacsia-block-vector (make-vector 256 nil)
  "A vector of the different blocks.")

(defvar emacsia-block-vector-blocks 0
  "The amount of blocks currently in `emacsia-block-vector'.")

(defstruct emacsia-block
  ;; Actual block-data
  obstaclep				; is this block an obstacle?
  display				; what to display instead?
  face					; what face should be used for it?
  permalit				; if the block is permanently lit
  opaque				; if the blocks blocks vision
  ;; Meta-data
  index					; for reverse indexing, the
					; index in
					; `emacsia-block-vector'
  )

(defun emacsia-block-add (name display &optional face obstacle permalit opaque)
  "Create and add a block NAME, displayed as DISPLAY.

DISPLAY will be in FACE if specified, the block will be an
OBSTACLE, if non-nil, and the same with PERMALIT. If OPAQUE is
non-nil, it will not be possible to see through it."
  (if (> emacsia-block-vector-blocks 255)
      ;; perhaps handle this in a better way?
      (error "Too many blocks allocated!")
    (let* ((newblocksym (intern (concat "emacsia-block-"
					(symbol-name name))))
	   (facesym (intern (concat "emacsia-block-"
				    (symbol-name name)
				    "-face")))
	   (newblock (make-emacsia-block
		      :obstaclep obstacle
		      :display (string-to-char display)
		      :index emacsia-block-vector-blocks
		      :permalit permalit
		      :opaque opaque
		      :face facesym)))
      (set newblocksym newblock)
      (make-face facesym)
      (set-face-attribute facesym nil :background "black")
      (apply 'set-face-attribute facesym nil face)
      (aset emacsia-block-vector emacsia-block-vector-blocks
	    newblock))
    (setq emacsia-block-vector-blocks
	  (+ emacsia-block-vector-blocks 1))))

(defun emacsia-block-load-blocks ()
  "Load the block-vector."
  (setq emacsia-block-vector (make-vector 256 nil)
	emacsia-block-vector-blocks 0)
  ;; Special blocks always at the same place in the display-table
  (emacsia-block-add 'unexamined     " " '(:foreground "black"))
  (emacsia-block-add 'player         "@" '(:foreground "yellow") t t)
  (emacsia-block-add 'permanent-wall "#" '(:foreground "white")  t t t)
  (emacsia-block-add 'granite-wall   "#" '(:foreground "gray")   t t t)
  (emacsia-block-add 'floor-lit      "." '(:foreground "white") nil t)
  (emacsia-block-add 'floor-unlit    "." '(:foreground "white"))
  (emacsia-block-add 'door-opened    "'" '(:foreground "khaki" :weight bold) nil t nil)
  (emacsia-block-add 'door-closed    "+" '(:foreground "khaki" :weight bold) t t))

(provide 'emacsia-block)
;;; emacsia-block.el ends here
