;;; emacsia-gg.el --- Gamegrid interface for Emacsia

;; Copyright (C) 2004  Free Software Foundation, Inc.

;; Author: Ulrik Jensen <terryp@daimi.au.dk>
;; Keywords: 

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:
(require 'emacsia-grid)

;; Buffer
(defvar emacsia-gg-buffer-name "*Emacsia*"
  "The name of the buffer used for playing Emacsia in.")

(defvar emacsia-gg-buffer-width 80
  "The width of the Emacsia-buffer.")

(defvar emacsia-gg-buffer-height 25
  "The height of the Emacsia-buffer.")

(defconst emacsia-gg-player-glyph 1
  "The entry in the display-table always used for the player.")

;; The grid
(defvar emacsia-gg-display-options (make-vector 256 nil)
  "The display-options used in the Emacsia-buffer.")

;; (defun emacsia-gg-define-glyph (char block &optional face)
;;   "Define the BLOCK to show with FACE instead of CHAR."
;;   (aset emacsia-gg-display-options
;; 	char
;; 	`(((t ,block))
;; 	  ((t color-x))
;; 	  ((t ,face)))))

(defun emacsia-gg-define-glyph (char block &optional face)
  "Define the BLOCK to show with FACE instead of CHAR."
  (aset emacsia-gg-display-options char `(:face ,face :display ,block)))

(defun emacsia-gg-make-display-options ()
  "Generate the data in `emacsia-gg-display-options' and return it."
  ;; so far, just set it based on `emacsia-block-vector'.
  (setq emacsia-gg-display-options (make-vector 256 nil))
  (dotimes (i emacsia-block-vector-blocks)
    (when (aref emacsia-block-vector i)
      (let ((curblock (eval (aref emacsia-block-vector i))))
	(emacsia-gg-define-glyph
	 i
	 (emacsia-block-display curblock)
	 (emacsia-block-face curblock)))))
  emacsia-gg-display-options)

;; interface
(defun emacsia-gg-init-buffer ()
  "Initialise a buffer for Emacsia."
  (switch-to-buffer (get-buffer-create emacsia-gg-buffer-name))
  (emacsia-gg-mode)
  (emacsia-gg-redraw))

(define-derived-mode emacsia-gg-mode fundamental-mode "Adventures of Emacsia"
  "A mode for playing Emacsia with Gamegrid."
  (setq header-line-format " [ The Adventures of Emacsia ]")
  (emacsia-grid-init (emacsia-gg-make-display-options)
		     emacsia-gg-buffer-width
		     emacsia-gg-buffer-height
		     (emacsia-block-index emacsia-block-unexamined)))

;;   (gamegrid-init (emacsia-gg-make-display-options))
;;   (gamegrid-init-buffer emacsia-gg-buffer-width
;; 			emacsia-gg-buffer-height
;; 			(emacsia-block-index emacsia-block-unexamined)))

(define-key emacsia-gg-mode-map (kbd "<up>")    'emacsia-gg-move-north)
(define-key emacsia-gg-mode-map (kbd "<down>")  'emacsia-gg-move-south)
(define-key emacsia-gg-mode-map (kbd "<left>")  'emacsia-gg-move-west)
(define-key emacsia-gg-mode-map (kbd "<right>") 'emacsia-gg-move-east)
(define-key emacsia-gg-mode-map (kbd "C-p")     'emacsia-gg-move-north)
(define-key emacsia-gg-mode-map (kbd "C-n")     'emacsia-gg-move-south)
(define-key emacsia-gg-mode-map (kbd "C-b")     'emacsia-gg-move-west)
(define-key emacsia-gg-mode-map (kbd "C-f")     'emacsia-gg-move-east)

(define-key emacsia-gg-mode-map (kbd "<home>")    'emacsia-gg-move-northwest)
(define-key emacsia-gg-mode-map (kbd "<next>")    'emacsia-gg-move-southeast)
(define-key emacsia-gg-mode-map (kbd "<prior>")   'emacsia-gg-move-northeast)
(define-key emacsia-gg-mode-map (kbd "<end>")     'emacsia-gg-move-southwest)
(define-key emacsia-gg-mode-map (kbd "*")         'emacsia-gg-set-targeting-mode)

(defun emacsia-gg-move-north ()
  "Move the player one square north."
  (interactive)
  (emacsia-move emacsia-direction-north))

(defun emacsia-gg-move-south ()
  "Move the player one square south."
  (interactive)
  (emacsia-move emacsia-direction-south))

(defun emacsia-gg-move-east ()
  "Move the player one square east."
  (interactive)
  (emacsia-move emacsia-direction-east))

(defun emacsia-gg-move-west ()
  "Move the player one square west."
  (interactive)
  (emacsia-move emacsia-direction-west))

(defun emacsia-gg-move-northwest ()
  "Move the player one square northwest."
  (interactive)
  (emacsia-move emacsia-direction-northwest))

(defun emacsia-gg-move-southwest ()
  "Move the player one square southwest."
  (interactive)
  (emacsia-move emacsia-direction-southwest))

(defun emacsia-gg-move-northeast ()
  "Move the player one square northeast."
  (interactive)
  (emacsia-move emacsia-direction-northeast))

(defun emacsia-gg-move-southeast ()
  "Move the player one square southeast."
  (interactive)
  (emacsia-move emacsia-direction-southeast))

(defun emacsia-gg-set-targeting-mode ()
  "Activate targetting mode."
  (interactive)
  ;; (emacsia-targeting-mode 1)
  )

(defun emacsia-gg-redraw ()
  "Redraw the entire screen with the current map."
  ;; Draw the basic level
  (let ((map (emacsia-level-visible emacsia-level-current)))
    (dotimes (y (length map))
      (dotimes (x (length (aref map y)))
	(emacsia-grid-set-cell x y (aref (aref map y) x))))
    ;; Now draw the visible entities on top
    ;; (get-visible-entities or so)
    (emacsia-grid-set-cell (emacsia-pos-x emacsia-current-position)
			   (emacsia-pos-y emacsia-current-position)
			   emacsia-gg-player-glyph)))

(defun emacsia-gg-redraw-pos (pos)
  "Redraw POS."
  (emacsia-grid-set-cell (emacsia-pos-x pos) (emacsia-pos-y pos)
			 (emacsia-block-index
			  (emacsia-level-get-visible-block pos))))

(defun emacsia-gg-update ()
  "Update the display after a cycle."
  ;; Loop through all updated positions and update them
  (mapc 'emacsia-gg-redraw-pos emacsia-touched)
  (emacsia-grid-set-cell (emacsia-pos-x emacsia-current-position)
			 (emacsia-pos-y emacsia-current-position)
			 emacsia-gg-player-glyph)  
  (setq emacsia-touched nil))

(provide 'emacsia-gg)
;;; emacsia-gg.el ends here
