;;; emacsia-grid.el --- Gamegrid rewrite

;; Copyright (C) 2004  Free Software Foundation, Inc.

;; Author: Ulrik Jensen <terryp@daimi.au.dk>
;; Keywords: 

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(defvar emacsia-grid-faces nil
  "A vector mapping characters to their faces.")

(defvar emacsia-grid-display nil
  "A display-table for the grid-buffer.")

(defvar emacsia-grid-buffer-height nil
  "Height of the grid-buffer.")

(defvar emacsia-grid-buffer-width nil
  "Width of the grid-buffer.")

(defvar emacsia-grid-buffer-start nil
  "The start-point of the grid-buffer.")

(defun emacsia-grid-init (displayoptions width height blank)
  "Initialise the current buffer for a grid.

DISPLAYOPTIONS should be a vector containing the options parsed into a
display table.

Each element in the vector should be a list of the form '(:face face
:display char)."
  (setq buffer-read-only t		; read only
	truncate-lines t
	cursor-type nil)		; disable the cursor	  
  (setq emacsia-grid-buffer-height height
	emacsia-grid-buffer-width  width
	emacsia-grid-buffer-start (point-min))
  ;; set up the buffers faces and display-table
  (setq emacsia-grid-faces (make-vector 256 nil))
  (setq emacsia-grid-display (make-display-table))
  (dotimes (i 256)
    (when (aref displayoptions i)
      (aset emacsia-grid-display i (vector (plist-get (aref displayoptions i)
						      :display)))
      (aset emacsia-grid-faces i (plist-get (aref displayoptions i)
					    :face))))
  ;; Activate the display-table
  (setq buffer-display-table emacsia-grid-display)
  ;; blank out the width x height
  (goto-char (point-min))
  (let ((line (concat (make-string width blank) "\n"))
	(buffer-read-only nil))
    (dotimes (i height)
      (insert line))
    ;; apply faces
    (add-text-properties (point-min) (point-max)
			 `(face ,(aref emacsia-grid-faces blank)))))

(defun emacsia-grid-offset (x y)
  "Return the POINT representing X,Y."
  (+ emacsia-grid-buffer-start
     (* (1+ emacsia-grid-buffer-width) y)
     x))

(defun emacsia-grid-set-face (char)
  "Set the face of point according to `emacsia-grid-faces' and CHAR."
  (put-text-property (1- (point)) (point)
		     'face (aref emacsia-grid-faces char)))

(defun emacsia-grid-set-cell (x y char)
  "Set the cell at position X,Y to CHAR."
  (let ((buffer-read-only nil))
    (goto-char (emacsia-grid-offset x y))
    (delete-char 1)
    (insert-char char 1)
    (emacsia-grid-set-face char)))

(provide 'emacsia-grid)
;;; emacsia-grid.el ends here
